@@ -1,5 +1,5 @@
 {-# LANGUAGE PackageImports #-}
-module Typed.Generic (benchmarks) where
+module Typed.Generic (benchmarks, decodeBenchmarks) where
 
 import Prelude ()
 import Prelude.Compat
@@ -7,7 +7,7 @@ import Prelude.Compat
 import "aeson" Data.Aeson hiding (Result)
 import Criterion
 import Data.ByteString.Lazy as L
-import Twitter.TH
+import Twitter.Generic
 import Typed.Common
 import qualified "aeson-benchmarks" Data.Aeson as B
 
@@ -26,7 +26,7 @@ encodeViaValueB = B.encode . B.toJSON
 benchmarks :: Benchmark
 benchmarks =
   env ((,) <$> load "json-data/twitter100.json" <*> load "json-data/jp100.json") $ \ ~(twitter100, jp100) ->
-  bgroup "generic" [
+  bgroup "encodeGeneric" [
       bgroup "direct" [
         bench "twitter100"          $ nf encodeDirectB twitter100
       , bench "jp100"               $ nf encodeDirectB jp100
@@ -40,3 +40,21 @@ benchmarks =
       , bench "jp100 baseline"      $ nf encodeViaValueA jp100
       ]
     ]
+
+decodeDirectA :: L.ByteString -> Maybe Result
+decodeDirectA = decode
+
+decodeDirectB :: L.ByteString -> Maybe Result
+decodeDirectB = B.decode
+
+decodeBenchmarks :: Benchmark
+decodeBenchmarks =
+  env ((,) <$> L.readFile "json-data/twitter100.json" <*> L.readFile "json-data/jp100.json") $ \ ~(twitter100, jp100) ->
+  bgroup "decodeGeneric"
+    [ bgroup "direct"
+      [ bench "twitter100"          $ nf decodeDirectB twitter100
+      , bench "jp100"               $ nf decodeDirectB jp100
+      , bench "twitter100 baseline" $ nf decodeDirectA twitter100
+      , bench "jp100 baseline"      $ nf decodeDirectA jp100
+      ]
+    ]
